from rest_framework import serializers
from django.contrib.auth.models import User
from models import Post, UserProfile


class UserSerializer(serializers.ModelSerializer):
    posts = serializers.HyperlinkedIdentityField(view_name='userpost-list', lookup_field='username')

    class Meta:
        model = User
        fields = ('id', 'username', 'first_name', 'last_name', 'posts', )


class PostSerializer(serializers.ModelSerializer):
    author = UserSerializer(required=False)

    def get_validation_exclusions(self, *args, **kwargs):
        # Need to exclude `user` since we'll add that later based off the request
        exclusions = super(PostSerializer, self).get_validation_exclusions(*args, **kwargs)
        return exclusions + ['author']

    class Meta:
        model = Post


class UserProfileSerializer(serializers.ModelSerializer):
    follows = UserSerializer(many=True, read_only=True)

    class Meta:
        model = UserProfile



import json
from django.http import JsonResponse

from rest_framework import generics, permissions
from rest_framework.response import Response
from rest_framework import status
from rest_framework.decorators import api_view


from serializers import UserSerializer, PostSerializer, UserProfileSerializer
from django.contrib.auth.models import User
from django.core.exceptions import ObjectDoesNotExist
from models import Post, UserProfile
from permissions import PostAuthorCanEditPermission

from allauth.socialaccount.providers.facebook.views import FacebookOAuth2Adapter
from rest_auth.registration.views import SocialLogin


class FacebookLogin(SocialLogin):
    adapter_class = FacebookOAuth2Adapter


class UserList(generics.ListCreateAPIView):
    queryset = User.objects.all()
    serializer_class = UserSerializer
    permission_classes = [
        permissions.AllowAny
    ]


class UserDetail(generics.RetrieveAPIView):
    queryset = User.objects.all()
    serializer_class = UserSerializer
    lookup_field = 'username'


class PostMixin(object):
    queryset = Post.objects.all()
    serializer_class = PostSerializer
    permission_classes = [
        PostAuthorCanEditPermission
    ]

    def pre_save(self, obj):
        """Force author to the current user on save"""
        obj.author = self.request.user
        return super(PostMixin, self).pre_save(obj)


class PostList(PostMixin, generics.ListCreateAPIView):
    pass


class PostDetail(PostMixin, generics.RetrieveUpdateDestroyAPIView):
    pass


class UserPostList(generics.ListAPIView):
    queryset = Post.objects.all()
    serializer_class = PostSerializer

    def get_queryset(self):
        queryset = super(UserPostList, self).get_queryset()
        return queryset.filter(author__username=self.kwargs.get('username'))


@api_view(['POST'])
def add_post(request):
    # POST request handler
    if request.method == 'POST':
        data = json.loads(request.body)
        author = request.user

        post = Post(text=data['text'], author=author)
        post.save()

        return Response(status=status.HTTP_201_CREATED)


@api_view(['POST'])
def follow(request):
    # POST request handler
    if request.method == 'POST':
        data = json.loads(request.body)

        follower = request.user
        follows = User.objects.get(username=data['follows'])

        try:
            follower_profile = UserProfile.objects.get(user=follower)
        except ObjectDoesNotExist:
            follower_profile = UserProfile(user=follower)
            follower_profile.save()

        try:
            follows_profile = UserProfile.objects.get(user=follows)
        except ObjectDoesNotExist:
            follows_profile = UserProfile(user=follows)
            follows_profile.save()

        follower_profile.follows.add(follows_profile)

        return Response(status=status.HTTP_201_CREATED)


@api_view(['POST'])
def unfollow(request):
    # POST request handler
    if request.method == 'POST':
        data = json.loads(request.body)

        follower = request.user
        unfollow_user = User.objects.get(username=data['unfollow'])

        follower_profile = UserProfile.objects.get(user=follower)
        unfollow_profile = UserProfile.objects.get(user=unfollow_user)

        follower_profile.follows.remove(unfollow_profile)

        return Response(status=status.HTTP_204_NO_CONTENT)


@api_view(['POST'])
def following(request):
    # POST request handler
    if request.method == 'POST':
        follower = request.user

        if str(follower) == 'AnonymousUser':
            # Makes sense only when user is logged in.
            return JsonResponse([], safe=False)

        try:
            follower_profile = UserProfile.objects.get(user=follower)
        except ObjectDoesNotExist:
            follower_profile = UserProfile(user=follower)
            follower_profile.save()

        following_list = follower_profile.follows.all()

        following_usernames = []

        for following in following_list:
            current_user = following.user
            following_usernames.append(current_user.username)

        return JsonResponse(following_usernames, safe=False)



from rest_framework import permissions


class SafeMethodsOnlyPermission(permissions.BasePermission):
    """Only can access non-destructive methods (like GET and HEAD)"""
    def has_permission(self, request, view):
        return self.has_object_permission(request, view)

    def has_object_permission(self, request, view, obj=None):
        return request.method in permissions.SAFE_METHODS


class PostAuthorCanEditPermission(SafeMethodsOnlyPermission):
    """Allow everyone to list or view, but only the author can modify existing instances"""
    def has_object_permission(self, request, view, obj=None):
        if obj is None:
            # Either a list or a create, so no author
            can_edit = True
        else:
            can_edit = request.user == obj.author
        return can_edit or super(PostAuthorCanEditPermission, self).has_object_permission(request, view, obj)


try:
    import json
except ImportError:
    import simplejson as json
from django.test import TestCase
from django.contrib.auth.models import User


class TestNucleoApi(TestCase):
    '''API TESTs'''

    fixtures = ['user.json', 'post.json', 'userprofile.json']

    def setUp(self):
        self.mock_user = {
            'email': 'g0ph@g0ph.com',
            'name': 'g0ph',
            'password': 'g0ph',
        }
        # User.objects.create_user(self.mock_user['name'],
        #                          self.mock_user['email'],
        #                          self.mock_user['password'])
        self.client.login(
            username=self.mock_user['name'],
            password=self.mock_user['password'],
        )
        # Each self.client.get show pass follow=True

    def test_following(self):
        # test a normal "logged-in" situation
        response = self.client.post('/api/following', {})
        self.assertEqual(response.status_code, 200)
        following = json.loads(response.content)
        assert 'user1' in following
        # Log out and check what comes out
        self.client.logout()
        response = self.client.post('/api/following', {})
        self.assertEqual(response.status_code, 200)
        following = json.loads(response.content)
        assert len(following) == 0




#!/usr/bin/env python
import os
import sys

from setuptools import find_packages, setup

EXCLUDE_FROM_PACKAGES = ['nucleo.migrations', 'nucleo.fixtures']
version = __import__('nucleo').__version__


def get_required():
    required_txt = os.path.join(os.path.dirname(__file__), 'requirements.txt')
    return open(required_txt).readlines()

extras = {
    'develop': [
        'nose',
    ]
}

setup(
    name='nucleo',
    version=version,
    url='https://github.com/Naible/django-nucleo/',
    author='Naible',
    author_email='admin@naible.com',
    description=('A django app for building social networks.'),
    license='GPLv3+',
    packages=find_packages(exclude=EXCLUDE_FROM_PACKAGES),
    include_package_data=True,
    install_requires=get_required(),
    extras_require=extras,
    zip_safe=False,
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        # 'Programming Language :: Python :: 3',
        # 'Programming Language :: Python :: 3.3',
        # 'Programming Language :: Python :: 3.4',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Internet :: WWW/HTTP :: WSGI',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)




from django.db import models
from django.contrib.auth.models import User

from datetime import datetime


# Post model
class Post(models.Model):
    date = models.DateTimeField(default=datetime.now)
    text = models.CharField(max_length=250)
    author = models.ForeignKey(User)


#  UserProfile model
class UserProfile(models.Model):
    user = models.OneToOneField(User, primary_key=True)
    follows = models.ManyToManyField('self', related_name='followed_by',
                                     symmetrical=False)

    def __unicode__(self):
        return self.user.username
